package net.sf.statscm;

/*
 * Copyright 2006 Doug Culnane
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import net.sf.statsvn.util.SvnStartupUtils;
import net.sf.statsvn.util.SvnVersionMismatchException;
import net.sf.statcvs.Messages;
import net.sf.statcvs.input.LogSyntaxException;
import net.sf.statcvs.output.CommandLineParser;
import net.sf.statsvn.output.SvnCommandLineParser;
import net.sf.statcvs.output.ConfigurationException;
import org.apache.maven.doxia.sink.Sink;
import org.apache.maven.doxia.siterenderer.Renderer;
import org.apache.maven.reporting.AbstractMavenReport;
import org.apache.maven.reporting.MavenReportException;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Locale;
import java.util.ResourceBundle;

import org.apache.maven.project.MavenProject;

/**
 * Generate a Source Code Management Metrics Report.
 * 
 * @goal stats
 * @phase site
 */
public class StatScmMojo extends AbstractMavenReport
{

    /**
     * Maven Project. This represents the POM where StatSCM gets most of its configuration from. This object is provided
     * by Maven.
     * 
     * @parameter expression="${project}"
     * @required
     * @readonly
     */
    private MavenProject project;

    /**
     * All messages for i18n.
     */
    private static ResourceBundle resourceBundle =
        ResourceBundle.getBundle( "net.sf.statscm.message", java.util.Locale.ENGLISH );

    /**
     * For running directly.
     * @todo This direct call should generate HTML reports by changing the output from Xdoc to html and the outputDir.
     */
    public void execute()
    {
        getLog().error( "StatSCM can not be run directly!" );
        getLog().info( "Configure it to run with the \"mvn site\" command." );
        getLog().info( "See: http://sourceforge.net/tracker/index.php?func=detail&aid=1610964&group_id=182522&atid=901553." );
    }

    /**
     * Main mojo execution method called by plugin environment.
     * 
     * @param loc Locale fro 118n.
     * @throws MavenReportException
     *             Throws an error if any failure.
     */
    public void executeReport( Locale loc ) throws MavenReportException
    {

        // re initialise Resource bundle wtih local
        resourceBundle = ResourceBundle.getBundle( "net.sf.statscm.message", loc );

        // Start
        getLog().info( "StatSCM Generating Source Code Management  Metrics." );
        
        // Configure
        try
        {
            getLog().info( "Configuring StatXXX" );
            StatConf.configure( project );
            getLog().info( "SCM Connection Type :" + StatConf.getConnectionType() );
            getLog().info( "Output Directory    :" + this.getOutputDirectory() );
        }
        catch ( ConfigurationException e )
        {
            getLog().error( resourceBundle.getString( "error.config" ), e );
            throw new MavenReportException( resourceBundle.getString( "error.config" ) );
        }

        // Get the Log file
        SrcManager scm = new SrcManager();
        if ( !scm.log( StatConf.getBaseDirectory(), getLog() ) )
        {
            throw new MavenReportException( resourceBundle.getString( "error.scmlog.file_missing" ) );
        }

        // Create Output dirs
        if ( !createOutputDirectory() )
        {
            throw new MavenReportException( resourceBundle.getString( "error.ouput.can_not_create_output_folder" ) );
        }

        // Perform report
        if ( StatConf.isStatSVN() || StatConf.isStatCVS() )
        {
            if ( StatConf.isStatSVN() )
            {
                doSvnStats();
            }
            else if ( StatConf.isStatCVS() )
            {
                doCvsStats();
            }
            doGenerateReport( getSink() );
            copyResourceFiles( getOutputDirectory(), project.getBasedir() + StatConf.FILE_SEPARATOR + "target"
                            + StatConf.FILE_SEPARATOR + "site" + StatConf.FILE_SEPARATOR + StatConf.STATSCM_DIR_NAME );
        }
        else
        {
            getLog().error( resourceBundle.getString( "warn.scm.type_xxx" ) );
        }
    }

    /**
     * Use StatSVN to generate xDoc output.
     * 
     * @throws MavenReportException if there is an error.
     */
    private void doSvnStats() throws MavenReportException
    {
        Messages.setPrimaryResource( "net.sf.statsvn.statcvs" );
        try
        {
            Messages.setPrimaryResource( "net.sf.statsvn.statcvs" );
            new SvnCommandLineParser( new String[] { StatConf.getLogFileName(), "." } ).parse();
            SvnStartupUtils.checkSvnVersionSufficient();
            SvnStartupUtils.checkRepoRootAvailable();
            net.sf.statsvn.Main.generateDefaultHTMLSuite();
        }
        catch ( LogSyntaxException e )
        {
            getLog().error( resourceBundle.getString( "error.scmlog.parsing" ), e );
            throw new MavenReportException( e.getMessage() );
        }
        catch ( IOException e )
        {
            getLog().error( "Error generating Subversion Stats.", e );
            throw new MavenReportException( e.getMessage() );
        }
        catch ( ConfigurationException e )
        {
            getLog().error( resourceBundle.getString( "error.config" ), e );
            throw new MavenReportException( e.getMessage() );

        }
        catch ( SvnVersionMismatchException e )
        {
            getLog().error( e.getMessage(), e );
            throw new MavenReportException( e.getMessage() );
        }
        catch ( NullPointerException e )
        {
        	getLog().warn("Null Pointer: Sometimes happens when local files are not synced with the repository!");
            getLog().error( e.getMessage(), e );
            //throw new MavenReportException( e.getMessage() );
        }
    }

    /**
     * Use StatCVS to generate xDoc output.
     * 
     * @throws MavenReportException if there is an error.
     */
    private void doCvsStats() throws MavenReportException
    {

        try
        {
            new CommandLineParser( new String[] { StatConf.getLogFileName(), "." } ).parse();
            net.sf.statcvs.Main.generateDefaultHTMLSuite();
        }
        catch ( LogSyntaxException e )
        {
            getLog().error( resourceBundle.getString( "error.scmlog.parsing" ), e );
            throw new MavenReportException( e.getMessage() );
        }
        catch ( IOException e )
        {
            getLog().error( "Error generating Subversion Stats.", e );
            throw new MavenReportException( e.getMessage() );
        }
        catch ( ConfigurationException e )
        {
            getLog().error( resourceBundle.getString( "error.config" ), e );
            throw new MavenReportException( e.getMessage() );
        }
    }

    /**
     * Create the output directory for generated xdoc output.
     * 
     * @return True if directory there or was sucessfully created.
     */
    private boolean createOutputDirectory()
    {
        File outputDir = new File( getOutputDirectory() );
        if ( outputDir.exists() )
        {
            if ( outputDir.isDirectory() )
            {
                return true;
            }
            else
            {
                getLog().error(
                                resourceBundle.getString( "error.outputDir.file_in_the_way" )
                                                + outputDir.getAbsolutePath() );
                return false;
            }
        }
        else
        {
            if ( outputDir.mkdirs() )
            {
                return true;
            }
            else
            {
                getLog().error( "Can  not make output directory at: " + outputDir.getAbsolutePath() );
                return false;
            }
        }
    }

    /**
     * Called my Maven Repoerint API.
     * 
     * @return Null becuase we handel the rendering ourselves.
     */
    protected Renderer getSiteRenderer()
    {
        return null;
    }

    /**
     * String repesentation of the output generated XDOC Directory.
     * 
     * @return Absolute path to the generated XDOC Directory.
     */
    protected String getOutputDirectory()
    {
        return StatConf.getOutputDir();
    }

    /**
     * String name of the html file that the Reporting Menu uses for a link.
     * 
     * @return Name of Html file to link to.
     */
    public String getOutputName()
    {
        return StatConf.STATSCM_DIR_NAME + "/statscm";
    }

    /**
     * Menu Item name for Reporting API.
     * 
     * @param locale
     *            Locale for i18n.
     * @return link menu name.
     */
    public String getName( Locale locale )
    {
        return "StatSCM";
    }

    /**
     * Description of Report for Reporting API.
     * 
     * @param loc Locale fro 118n.
     * @return i18n Description of Report.
     */
    public String getDescription( Locale loc )
    {
        return resourceBundle.getString( "statscm.description" );
    }

    /**
     * Utility method for copying resource files.
     * 
     * @param src
     *            Source Directory.
     * @param dest
     *            Target Directory.
     */
    private void copyResourceFiles( String src, String dest )
    {

        File srcDir = new File( src );
        if ( !srcDir.isDirectory() )
        {
            getLog().error( "Can not copy reources src is not a directory: " + srcDir.getAbsolutePath() );
            return;
        }
        File destDir = new File( dest );
        if ( !destDir.isDirectory() )
        {
            destDir.mkdirs();
        }

        String[] resoures = srcDir.list( new java.io.FilenameFilter()
        {
            public boolean accept( File dir, String name )
            {
                String[] extensions = { ".png", ".jar", ".txt" };
                String fileName = name.toLowerCase( Locale.getDefault() );
                for ( int i = 0; i < extensions.length; i++ )
                {
                    if ( fileName.endsWith( extensions[i] ) )
                    {
                        return true;
                    }
                }
                return false;

            }
        } );

        for ( int i = 0; i < resoures.length; i++ )
        {
            FileInputStream fis = null;
            FileOutputStream fos = null;
            try
            {
                fis = new FileInputStream( new File( src + StatConf.FILE_SEPARATOR + resoures[i] ) );
                fos = new FileOutputStream( new File( dest + StatConf.FILE_SEPARATOR + resoures[i] ) );
                int b = -1;
                while ( ( b = fis.read() ) != -1 )
                {
                    fos.write( b );
                }

                fis.close();
                fos.close();
            }
            catch ( FileNotFoundException e )
            {
                getLog().error( "Can not find file " + resoures[i], e );
            }
            catch ( IOException e )
            {
                getLog().error( "Error copying file: " + src + "/" + resoures[i], e );
            }
            finally
            {
                try
                {
                    if ( fis != null )
                    {
                        fis.close();
                    }
                    if ( fos != null )
                    {
                        fos.close();
                    }
                }
                catch ( IOException e )
                {
                    getLog().error( "I/O Exception copying resources.", e );
                }
                finally
                {
                    fis = null;
                    fos = null;
                }
            }
            if ( getLog().isDebugEnabled() )
            {
                getLog().debug(
                                "Copied file " + src + StatConf.FILE_SEPARATOR + resoures[i] + " to " + dest
                                                + StatConf.FILE_SEPARATOR + resoures[i] );
            }
        }
    }

    /**
     * Create a structured StatSCM / StatSVN report front page.
     * 
     * @param sink
     *            used to write structured report to.
     */
    private void doGenerateReport( Sink sink )
    {

        // protect against direct runs.
        if ( sink == null )
        {
            return;
        }

        sink.head();
        sink.title();
        sink.text( "StatSCM" );
        sink.title_();
        sink.head_();

        sink.body();
        sink.section1();

        sink.sectionTitle1();
        sink.text( getDescription( Locale.getDefault() ) );
        sink.sectionTitle1_();

        sink.bold();
        sink.link( "index.html" );
        sink.text( "Main page" );
        sink.link_();
        sink.bold_();
        sink.link_();
        sink.lineBreak();

        // TODO Think about adding first page links this back in once the StatSVN and StatCVS output stableises.

        sink.link( "index.html" );
        sink.figure();
        sink.figureGraphics( "loc.png" );
        sink.figure_();
        sink.link_();
        sink.lineBreak();

        sink.text( "Generated using " );
        sink.link( "http://stat-scm.sourceforge.net/" );
        sink.text( "StatSCM" );
        sink.link_();
        sink.text( "." );

        sink.flush();
        sink.close();
    }

    /**
     * Access to the Maven Project representation for Reporting API.
     * 
     * @return MavenProject.
     */
    protected MavenProject getProject()
    {
        return project;
    }

    /**
     * Retrive a central Resouce bundle for i18n.
     * 
     * @return i18n resource bundle.
     */
    public static ResourceBundle getResourceBundle()
    {
        return resourceBundle;
    }

    /**
     * Setter to allow unit tests to set up Mojo.
     * 
     * @param project Maven Project
     */
    public void setProject( MavenProject project )
    {
        this.project = project;
    }
}
